#include <avr/io.h>
#include <avr/pgmspace.h>
#include <ow.h>


// Globale Variablen ///////////////////////////////////////////////////////////////////////////////////////////////////

 static unsigned long time_base;

 static unsigned short us500;
 static unsigned short us420;
 static unsigned short us120;
 static unsigned short us70;
 static unsigned short us12;
 static unsigned short us2;

 volatile unsigned char *PORT_OW;
 volatile unsigned char *DDR_OW;
 volatile unsigned char *PIN_OW;
 volatile unsigned char OW0;
 volatile unsigned char OW1;
 volatile unsigned char OW;
 
 static unsigned char ROM[8];
 static unsigned char lastDiscrep =0;
 static unsigned char doneFlag =0;
 static	unsigned char numROMs;
 static unsigned char dowcrc;
 prog_char owcrc_table[]={
 0,94,188,226,97,63,221,131,194,156,126,32,163,253,31,65,157,195,33,127,252,162,64,30,95,1,227,189,62,96,130,220,
 35,125,159,193,66,28,254,160,225,191,93,3,128,222,60,98,190,224,2,92,223,129,99,61,124,34,192,158,29,67,161,255,
 70,24,250,164,39,121,155,197,132,218,56,102,229,187,89,7,219,133,103,57,186,228,6,88,25,71,165,251,120,38,196,154,
 101,59,217,135,4,90,184,230,167,249,27,69,198,152,122,36,248,166,68,26,153,199,37,123,58,100,134,216,91,5,231,185,
 140,210,48,110,237,179,81,15,78,16,242,172,47,113,147,205,17,79,173,243,112,46,204,146,211,141,111,49,178,236,14,80,
 175,241,19,77,206,144,114,44,109,51,209,143,12,82,176,238,50,108,142,208,83,13,239,177,240,174,76,18,145,207,45,115,
 202,148,118,40,171,245,23,73,8,86,180,234,105,55,213,139,87,9,235,181,54,104,138,212,149,203,41,119,244,170,72,22,
 233,183,85,11,136,214,52,106,43,117,151,201,74,20,246,168,116,42,200,150,21,75,169,247,182,232,10,84,215,137,107,53};

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

// Prototypen///////////////////

unsigned char 	ow_first(unsigned char RomCommand);
unsigned char 	ow_next(unsigned char RomCommand);

///////////////////////////////

/******************************************************************************
* bergibt der OW_Schnittstelle	den Port Portpin und Quarzfrequenz      
*			      
******************************************************************************/
void _ow_init(volatile unsigned char *owport,unsigned char owpin,unsigned long f_cpu)
{
time_base = f_cpu/4000;

us500 = (time_base *500)/1000;
us420 = (time_base *420)/1000;
us120 = (time_base *120)/1000;
us70  = (time_base *70)/1000;
us12  = (time_base *12)/1000;
us2   = (time_base *2)/1000;

PORT_OW =  owport;
DDR_OW  =  owport-1;
PIN_OW  =  owport-2;
OW1     =  (1<<owpin);
OW0     = ~(1<<owpin);
OW      =  owpin;
}



/******************************************************************************
* Warteschleife in Mikrosekundenschritten bei 4MHZ	      
* der Aufruf dauert noch mal 1,5s			      
******************************************************************************/
#define ow_delay(x)    ({  unsigned short number_of_loops=(unsigned short)x;  \
                              __asm__ volatile ( "cp  %A0,__zero_reg__ \n\t"     \
                                                 "cpc %B0,__zero_reg__ \n\t"     \
                                                 "breq L_EXIT_%=       \n\t"     \
                                                 "L_%=:                \n\t"     \
                                                 "sbiw r24,1           \n\t"     \
                                                 "brne L_%=            \n\t"     \
                                                 "L_EXIT_%=:           \n\t"     \
                                                 : /* NO OUTPUT */               \
                                                 : "w" (number_of_loops)         \
                                               );                                \
                          })        


/******************************************************************************
* OW_Reset
* Reseted den OW_Bus und gibt das Presencesignal zurck
* 0 = presence , 1 = no part
******************************************************************************/
unsigned char ow_reset(void)
{
unsigned char x=0, global_int_mem=0;
if(SREG & 0x80) { asm volatile ("cli"); global_int_mem=1; }
*PORT_OW &=OW0;*DDR_OW |=OW1;			// OW_Bus auf low
ow_delay(us500);				// 500 s warten	
*DDR_OW &=OW0;					// OW_Bus als Eingang
ow_delay(us120);				// auf Presencesignal warten
if (bit_is_set(*PIN_OW,OW)) x=1;		// wenn der Bus high ist
ow_delay(us420);				// wird der wert 1 zurckgegeben sonst 0	
 						// zurckgeben und restliche Zeit warten
if(global_int_mem) { asm volatile ("sei"); }
return x;
}


/******************************************************************************
* READ_BIT
* liest ein Bit vom OW_Bus
******************************************************************************/
unsigned char ow_read_bit(void)
{
unsigned char x=0, global_int_mem=0;
if(SREG & 0x80) { asm volatile ("cli"); global_int_mem=1; }
*PORT_OW &=OW0;*DDR_OW |=OW1;			// OW_Bus auf low
ow_delay(us2);					// 2 s warten	
*DDR_OW &=OW0;					// OW_Bus als Eingang
ow_delay(us12);					// 12 s warten	
if (bit_is_set(*PIN_OW,OW)) x=1;		// wenn der Bus high ist wird	
ow_delay(us120);				// der wert 1 zurckgegeben	
						// sonst 0 zurckgeben
if(global_int_mem) { asm volatile ("sei"); }
return x;
}


/******************************************************************************
* WRITE_BIT
* schreibt ein Bit auf der OW_Bus
******************************************************************************/
void ow_write_bit(char bitval)
{
unsigned char global_int_mem=0;
if(SREG & 0x80) { asm volatile ("cli"); global_int_mem=1; }
*PORT_OW &=OW0;*DDR_OW |=OW1;			// OW_Bus auf low
ow_delay(us2);					// 2 s warten	
if (bitval==1) *DDR_OW &=OW0;			// OW_Bus als Eingang
ow_delay(us70);
*DDR_OW &=OW0;					// OW_Bus als Eingang
ow_delay(us12);					// 12 s warten	
if(global_int_mem) { asm volatile ("sei"); }
}



/******************************************************************************
* READ_BYTE
* liest ein BYTE vom OW_Bus
******************************************************************************/
unsigned char ow_read_byte(void)
{
unsigned char i;
unsigned char value = 0;
for (i=0;i<8;i++)
{
if (ow_read_bit()) value |= 0x01 <<i;
}
return(value);
}




/******************************************************************************
* WRITE_BYTE
* schreibt ein Byte auf den OW_bus
******************************************************************************/
void ow_write_byte(char val)
{
unsigned char i;
unsigned char temp;
for (i=0;i<8;i++)
{
	temp = val>>i;
	temp &= 0x01;
	ow_write_bit(temp);	
}
}



/******************************************************************************
* READ_WORD
* liest eim Word vom OW_Bus
******************************************************************************/
unsigned int ow_read_word(void)
{
unsigned char i;
unsigned int value = 0;
for (i=0;i<16;i++)
{
if (ow_read_bit()) value |= 0x01 <<i;
}
return(value);
}



/******************************************************************************
* FIND_DEVICE
* 
******************************************************************************/
unsigned char ow_search(unsigned char RomCommand,unsigned char FoundROM[][8],unsigned char family)
{
	unsigned char m;
	if (!ow_reset())			// Beginnt wenn Presence erkannt
	{
		if(ow_first(RomCommand))	// Beginnt wenn der letzte
		{				// Sensor gefunden ist
			numROMs=0;
		
	do		
			{
			for (m=0;m<8;m++)
			{
			FoundROM[numROMs][m]=ROM[m];  // ROM Indentifizieren
			}
			if (ROM[0] == family)	numROMs++ ;
			}while (ow_next(RomCommand)&&(numROMs<10));
		}
	}

return numROMs;	
}




/******************************************************************************
* FIRST
* 
******************************************************************************/
unsigned char ow_first(unsigned char RomCommand)
{
lastDiscrep=0;
doneFlag= FALSE;
return ow_next(RomCommand);
}



/******************************************************************************
* NEXT
* 
******************************************************************************/
unsigned char ow_next(unsigned char RomCommand)
{
unsigned char m = 1;
unsigned char n = 0;
unsigned char k = 1;
unsigned char x = 0;
unsigned char discrepMarker = 0;
unsigned char g ;
unsigned char nxt ;
int flag;

nxt = FALSE;
dowcrc = 0;

flag = ow_reset();
if(flag||doneFlag)
{
lastDiscrep = 0;
return FALSE;
}

ow_write_byte(RomCommand);		// send SearchROM command

do
{
x = 0;
if(ow_read_bit()==1)x=2;
if(ow_read_bit()==1)x|=1;
if(x==3)
break;

else
{
if(x>0)
g=x>>1;
else
{
if(m<lastDiscrep)
	g=((ROM[n]&k)>0);
	else
	g=(m==lastDiscrep);
	
	if(g==0)discrepMarker=m;
	}
	
	if(g==1)ROM[n]|=k;
	else ROM[n]&=~k;
	
	ow_write_bit(g);
	
m++;
k=k<<1;
if(k==0)
{
ow_crc(ROM[n]);
n++;k++;
}

}

}while(n<8);
if(m<65||dowcrc)
lastDiscrep=0;
else
{
lastDiscrep=discrepMarker;
doneFlag=(lastDiscrep==0);
nxt=TRUE;
}
return nxt;
}



/******************************************************************************
* ONE WIRE CRC
* 
******************************************************************************/
unsigned char ow_crc(unsigned char x)
{
dowcrc=pgm_read_byte(&owcrc_table[dowcrc^x]);
return dowcrc;
}

/******************************************************************************
* ONE WIRE reset_dowcrc
* 
******************************************************************************/
void ow_reset_dowcrc(void)
{
dowcrc=0;
}

/******************************************************************************
* MATCH ROM
* 
******************************************************************************/
unsigned char ow_match_rom(unsigned char FoundROM[][8],unsigned char numR)
{
unsigned char i;

if(ow_reset())return FALSE;
ow_write_byte(OW_MATCH_ROM);				// match ROM
for(i=0;i<8;i++)
{
ow_write_byte(FoundROM[numR][i]);			// sende ROM Code
}
return TRUE;
}


/******************************************************************************
* SKIP ROM
* 
******************************************************************************/
unsigned char ow_skip_rom(void)
{
if(ow_reset())return FALSE;
ow_write_byte(OW_SKIP_ROM);
return TRUE;
}


/******************************************************************************
* READ ROM
* 
******************************************************************************/
unsigned char ow_read_rom(unsigned char FoundROM[8])
{
unsigned char i;
if(ow_reset())return FALSE;
ow_write_byte(OW_READ_ROM);
for(i=0;i<8;)
{
FoundROM[i] = ow_read_byte();
i++;
}
return TRUE;
}

/******************************************************************************
* OW READY
Die Funktion ow ready() gibt eine Null(0) zurck, wenn vom One Wire Gert
innerhalb der Zeitschleife von 1,5 Sekunden keine Bereitschaft angezeigt wurde. Ansonsten
wird ein Wert ungleich Null(!=0) fr einen erfolgreichen Bereitschaftstest
zurckgegeben.
* 
******************************************************************************/
unsigned char ow_ready(void)
{
volatile unsigned short ms10 = (time_base *10000)/1000; 

	unsigned char timeout = 150; /* times 10ms * 150 = 1,5 second time out */
	while (!ow_read_bit() && timeout > 0){ ow_delay(ms10);timeout--; }
	return timeout;
}


